(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('rxjs/Subject')) :
	typeof define === 'function' && define.amd ? define(['rxjs/Subject'], factory) :
	(global.RxJS_OO = factory(global.Subject));
}(this, (function (Subject) { 'use strict';

Subject = Subject && Subject.hasOwnProperty('default') ? Subject['default'] : Subject;

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var observableFunction = createCommonjsModule(function (module, exports) {
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });

var ObservableFunction = (function () {
    function ObservableFunction(fn, parentEvents) {
        var onApply;
        if (parentEvents) {
            onApply = parentEvents.onApply;
        }
        else {
            onApply = new Subject.Subject();
        }
        this.proxy = new Proxy(fn, {
            apply: function (target, thisArg, argumentsList) {
                var result = target.apply(thisArg, argumentsList);
                onApply.next({ thisArg: thisArg, argumentsList: argumentsList, result: result, target: target });
                return result;
            }
        });
        this.events = {
            onApply: onApply,
        };
    }
    return ObservableFunction;
}());
exports.ObservableFunction = ObservableFunction;

});

unwrapExports(observableFunction);

var observableObject = createCommonjsModule(function (module, exports) {
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });


var ObservableObject = (function () {
    function ObservableObject(object, wrapProxiableProperties, parentEvents) {
        if (object === void 0) { object = {}; }
        if (wrapProxiableProperties === void 0) { wrapProxiableProperties = true; }
        var onGet;
        var onSet;
        var onDelete;
        var onApply;
        if (parentEvents) {
            (_a = parentEvents, onGet = _a.onGet, onSet = _a.onSet, onDelete = _a.onDelete);
        }
        else {
            onGet = new Subject.Subject();
            onSet = new Subject.Subject();
            onDelete = new Subject.Subject();
            onApply = new Subject.Subject();
        }
        var proxy = new Proxy(object, {
            get: function (target, property, receiver) {
                var value = target[property];
                onGet.next({ property: property, value: value, target: target });
                return value;
            },
            set: function (target, property, newValue, receiver) {
                var oldValue = target[property];
                target[property] = newValue;
                onSet.next({ property: property, oldValue: oldValue, newValue: newValue, target: target });
                return true;
            },
            deleteProperty: function (target, property) {
                var value = target[property];
                var ret = delete target[property];
                onDelete.next({ property: property, value: value, target: target });
                return ret;
            },
        });
        this.events = {
            onGet: onGet,
            onSet: onSet,
            onDelete: onDelete,
            onApply: onApply,
        };
        if (wrapProxiableProperties) {
            for (var _i = 0, _b = Object.keys(object); _i < _b.length; _i++) {
                var propertyName = _b[_i];
                var property = object[propertyName];
                if (typeof property === 'object') {
                    var nestedProxy = new ObservableObject(property, true, parentEvents ? parentEvents : this.events);
                    proxy[propertyName] = nestedProxy.proxy;
                }
                else if (typeof property === 'function') {
                    var proxyFunc = new observableFunction.ObservableFunction(property, parentEvents ? parentEvents : this.events).proxy;
                    proxy[propertyName] = proxyFunc;
                }
            }
        }
        this.proxy = proxy;
        var _a;
    }
    return ObservableObject;
}());
exports.ObservableObject = ObservableObject;

});

unwrapExports(observableObject);

var _package = createCommonjsModule(function (module, exports) {
"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
__export(observableObject);
__export(observableFunction);

});

var index = unwrapExports(_package);

return index;

})));
//# sourceMappingURL=rxjs-observable-object.js.map
