"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Subject_1 = require("rxjs/Subject");
var observable_function_1 = require("./observable-function");
var ObservableObject = (function () {
    function ObservableObject(object, wrapProxiableProperties, parentEvents) {
        if (object === void 0) { object = {}; }
        if (wrapProxiableProperties === void 0) { wrapProxiableProperties = true; }
        var onGet;
        var onSet;
        var onDelete;
        var onApply;
        if (parentEvents) {
            (_a = parentEvents, onGet = _a.onGet, onSet = _a.onSet, onDelete = _a.onDelete);
        }
        else {
            onGet = new Subject_1.Subject();
            onSet = new Subject_1.Subject();
            onDelete = new Subject_1.Subject();
            onApply = new Subject_1.Subject();
        }
        var proxy = new Proxy(object, {
            get: function (target, property, receiver) {
                var value = target[property];
                onGet.next({ property: property, value: value, target: target });
                return value;
            },
            set: function (target, property, newValue, receiver) {
                var oldValue = target[property];
                target[property] = newValue;
                onSet.next({ property: property, oldValue: oldValue, newValue: newValue, target: target });
                return true;
            },
            deleteProperty: function (target, property) {
                var value = target[property];
                var ret = delete target[property];
                onDelete.next({ property: property, value: value, target: target });
                return ret;
            },
        });
        this.events = {
            onGet: onGet,
            onSet: onSet,
            onDelete: onDelete,
            onApply: onApply,
        };
        if (wrapProxiableProperties) {
            for (var _i = 0, _b = Object.keys(object); _i < _b.length; _i++) {
                var propertyName = _b[_i];
                var property = object[propertyName];
                if (typeof property === 'object') {
                    var nestedProxy = new ObservableObject(property, true, parentEvents ? parentEvents : this.events);
                    proxy[propertyName] = nestedProxy.proxy;
                }
                else if (typeof property === 'function') {
                    var proxyFunc = new observable_function_1.ObservableFunction(property, parentEvents ? parentEvents : this.events).proxy;
                    proxy[propertyName] = proxyFunc;
                }
            }
        }
        this.proxy = proxy;
        var _a;
    }
    return ObservableObject;
}());
exports.ObservableObject = ObservableObject;
//# sourceMappingURL=observable-object.js.map